import operator
import hashlib
from importlib import import_module

from django.conf import settings
from django.contrib import auth
from django.core.urlresolvers import NoReverseMatch

from rest_framework import serializers, exceptions, relations
from rest_framework.reverse import reverse

from oscar.core.loading import get_class
import oscar.models.fields

Selector = get_class('partner.strategy', 'Selector')


def overridable(name, default):
    """
    Seems useless but this is for readability
    """
    return getattr(settings, name, default)


def expand_field_mapping(extra_fields):
    # This doesn't make a copy
    field_mapping = serializers.ModelSerializer.serializer_field_mapping
    field_mapping.update(extra_fields)
    return field_mapping


class OscarSerializer(object):
    field_mapping = expand_field_mapping({
        oscar.models.fields.NullCharField: serializers.CharField
    })

    def __init__(self, *args, **kwargs):
        """
        Allow the serializer to be initiated with only a subset of the
        speccified fields
        """
        fields = kwargs.pop('fields', None)
        super(OscarSerializer, self).__init__(*args, **kwargs)
        if fields:
            allowed = set(fields)
            existing = set(self.fields.keys())
            for field_name in existing - allowed:
                self.fields.pop(field_name)

    def to_native(self, obj):
        num_fields = len(self.get_fields())
        native = super(OscarSerializer, self).to_native(obj)

        if num_fields == 1:
            _, val = next(native.iteritems())
            return val

        return native


class OscarModelSerializer(OscarSerializer, serializers.ModelSerializer):
    """
    Correctly map oscar fields to serializer fields.
    """


class OscarHyperlinkedModelSerializer(
        OscarSerializer, serializers.HyperlinkedModelSerializer):
    """
    Correctly map oscar fields to serializer fields.
    """


class DrillDownHyperlinkedIdentityField(relations.HyperlinkedIdentityField):
    def __init__(self, *args, **kwargs):
        try:
            self.extra_url_kwargs = kwargs.pop('extra_url_kwargs')
        except KeyError:
            msg = "DrillDownHyperlinkedIdentityField requires 'extra_url_args' argument"
            raise ValueError(msg)

        super(DrillDownHyperlinkedIdentityField, self).__init__(*args, **kwargs)

    def get_extra_url_kwargs(self, obj):
        return {
            key: operator.attrgetter(path)(obj)
                for key, path in self.extra_url_kwargs.items()
        }

    def get_url(self, obj, view_name, request, format):
        """
        Given an object, return the URL that hyperlinks to the object.

        May raise a `NoReverseMatch` if the `view_name` and `lookup_field`
        attributes are not configured to correctly match the URL conf.
        """
        lookup_field = getattr(obj, self.lookup_field, None)
        kwargs = {self.lookup_field: lookup_field}
        kwargs.update(self.get_extra_url_kwargs(obj))
        # Handle unsaved object case
        if lookup_field is None:
            return None

        try:
            return reverse(view_name, kwargs=kwargs, request=request, format=format)
        except NoReverseMatch:
            pass

        if self.pk_url_kwarg != 'pk':
            # Only try pk lookup if it has been explicitly set.
            # Otherwise, the default `lookup_field = 'pk'` has us covered.
            kwargs = {self.pk_url_kwarg: obj.pk}
            kwargs.update(self.get_extra_url_kwargs(obj))
            try:
                return reverse(view_name, kwargs=kwargs, request=request, format=format)
            except NoReverseMatch:
                pass

        slug = getattr(obj, self.slug_field, None)
        if slug:
            # Only use slug lookup if a slug field exists on the model
            kwargs = {self.slug_url_kwarg: slug}
            kwargs.update(self.get_extra_url_kwargs(obj))
            try:
                return reverse(view_name, kwargs=kwargs, request=request, format=format)
            except NoReverseMatch:
                pass

        raise NoReverseMatch()


def get_domain(request):
    return request.get_host().split(':')[0]


def login_and_upgrade_session(request, user):
    "Upgrade anonymous session to authenticated session"
    parsed_session_uri = getattr(request, 'parsed_session_uri', None)

    if parsed_session_uri is not None:

        assert(parsed_session_uri['type'] == 'ANON')

        # change anonymous session to authenticated
        parsed_session_uri['type'] = "AUTH"
        session_id = session_id_from_parsed_session_uri(parsed_session_uri)

        # wipe out old anonymous session without creating a new one
        request.session.clear()
        request.session.delete()

        # start session with new session id
        request.session = get_session(session_id)

        # Mark the new session as owned by the user we are logging in.
        request.session[auth.SESSION_KEY] = user.pk
        if hasattr(user, 'get_session_auth_hash'):  # django 1.7
            request.session[auth.HASH_SESSION_KEY] = \
                user.get_session_auth_hash()

    # now login so the session can be used for authentication purposes.
    auth.login(request, user)
    request.session.save()


def session_id_from_parsed_session_uri(parsed_session_uri):
    session_id_base = u"SID:%(type)s:%(realm)s:%(session_id)s" % (
        parsed_session_uri)
    combined = session_id_base + settings.SECRET_KEY
    return hashlib.sha1(combined.encode()).hexdigest()


def get_session(session_id, raise_on_create=False):
    "get a session with the id specified."
    engine = import_module(settings.SESSION_ENGINE)
    session = engine.SessionStore(session_id)

    if not session.exists(session_id):
        if raise_on_create:
            raise exceptions.NotAuthenticated()
        else:
            session.save(must_create=True)
    else:
        # if we get an expired session from django,
        # the session key will change after calling load.
        # since the whole point of get_session is to retrieve a session
        # with axactly the key specified, we have to clear the expired
        # sessions and then get a new session.
        session.load()
        if session.session_key != session_id:
            engine.SessionStore.clear_expired()
            session = get_session(session_id, raise_on_create)

    return session



from importlib import import_module

from django.conf import settings

from oscarapi.tests.utils import APITest
from oscarapi.utils import get_session, session_id_from_parsed_session_uri


class LoginTest(APITest):

    def test_login_with_header(self):
        "Logging in with an anonymous session id header should upgrade to an authenticated session id"
        response = self.post('api-login', username='nobody', password='nobody', session_id='koe')

        self.assertEqual(response.status_code, 200)
        self.assertIn('Session-Id', response)
        self.assertEqual(response.get('Session-Id'), 'SID:AUTH:testserver:koe', 'the session type should be upgraded to AUTH')

        # check authentication worked
        with self.settings(DEBUG=True, OSCARAPI_USER_FIELDS=('username', 'id')):
            response = self.get('api-login', session_id='koe', authenticated=True)
            parsed_response = response.data

            self.assertEqual(parsed_response['username'], 'nobody')
            self.assertEqual(parsed_response['id'], 2)

        # note that this shows that we can move a session from one user to the
        # other! This is the responsibility of the client application!
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False, DEBUG=True, OSCARAPI_USER_FIELDS=('username', 'id')):
            response = self.post('api-login', username='admin', password='admin', session_id='koe')

            self.assertEqual(response.status_code, 200)
            self.assertIn('Session-Id', response)
            self.assertEqual(response.get('Session-Id'), 'SID:AUTH:testserver:koe', 'the session type should be upgraded to AUTH')

            # check authentication worked
            response = self.get('api-login', session_id='koe', authenticated=True)
            parsed_response = response.data

            self.assertEqual(response.status_code, 200)
            self.assertEqual(parsed_response['username'], 'admin')
            self.assertEqual(parsed_response['id'], 1)

    def test_failed_login_with_header(self):
        "Failed login should not upgrade to an authenticated session"

        response = self.post('api-login', username='nobody', password='somebody', session_id='koe')

        self.assertEqual(response.status_code, 401)
        self.assertIn('Session-Id', response)
        self.assertEqual(response.get('Session-Id'), 'SID:ANON:testserver:koe', 'the session type should NOT be upgraded to AUTH')

        # check authentication didn't work
        with self.settings(DEBUG=True, OSCARAPI_USER_FIELDS=('username', 'id')):
            response = self.get('api-login')
            self.assertFalse(response.content)
            self.assertEqual(response.status_code, 204)

    def test_login_without_header(self):
        "It should be possible to login using the normal cookie session"

        response = self.post('api-login', username='nobody', password='nobody')

        self.assertEqual(response.status_code, 200)
        self.assertEqual(int(self.client.session['_auth_user_id']), 2)
        self.assertNotIn('Session-Id', response)

        # check authentication worked
        with self.settings(DEBUG=True, OSCARAPI_USER_FIELDS=('username', 'id')):
            response = self.get('api-login')
            parsed_response = response.data

            self.assertEqual(parsed_response['username'], 'nobody')
            self.assertEqual(parsed_response['id'], 2)

        # using cookie sessions it is not possible to pass 1 session to another
        # user
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False, DEBUG=True, OSCARAPI_USER_FIELDS=('username', 'id')):
            response = self.post('api-login', username='admin', password='admin')

            self.assertEqual(response.status_code, 405)
            self.assertEqual(int(self.client.session['_auth_user_id']), 2)
            self.assertNotIn('Session-Id', response)

            # check we are still authenticated as nobody
            response = self.get('api-login')
            parsed_response = response.data

            self.assertEqual(parsed_response['username'], 'nobody')
            self.assertEqual(parsed_response['id'], 2)

    def test_logged_in_users_can_not_login_again_via_the_api(self):
        "It should not be possible to move a cookie session to a header session"
        self.login(username='nobody', password='nobody')
        first_key = self.client.session.session_key
        response = self.post('api-login', username='nobody', password='nobody')
        self.assertEqual(response.status_code, 405)
        self.assertEqual(self.client.session.session_key, first_key, 'Since login failed, the user should have the same session')

    def test_logging_out_with_header(self):
        "After logging out, a user can not use the session id to authenticate anymore"
        with self.settings(DEBUG=True):
            engine = import_module(settings.SESSION_ENGINE)
            session = engine.SessionStore()

            self.test_login_with_header()

            parsed_session_uri = {
                'realm': 'testserver',
                'type': 'AUTH',
                'session_id': 'koe'
            }
            session_id = session_id_from_parsed_session_uri(parsed_session_uri)
            self.assertTrue(session.exists(session_id))

            response = self.delete('api-login', session_id='koe', authenticated=True)

            self.assertFalse(session.exists(session_id))
            self.assertNotIn('Session-Id', response)

            response = self.get('api-login', session_id='koe', authenticated=True)
            self.assertEqual(response.status_code, 401)

    def test_logging_out_anonymous(self):
        "After logging out, an anonymous user can not use the session id to authenticate anymore"
        with self.settings(DEBUG=True, SESSION_SAVE_EVERY_REQUEST=True):
            engine = import_module(settings.SESSION_ENGINE)
            session = engine.SessionStore()

            # get a session running
            response = self.get('api-login', session_id='koe')
            parsed_session_uri = {
                'realm': 'testserver',
                'type': 'ANON',
                'session_id': 'koe'
            }
            session_id = session_id_from_parsed_session_uri(parsed_session_uri)
            self.assertTrue(session.exists(session_id))
            self.assertEqual(response.status_code, 204)

            # delete the session
            response = self.delete('api-login', session_id='koe')

            self.assertEqual(response.status_code, 200)
            self.assertFalse(session.exists(session_id))
            self.assertNotIn('Session-Id', response)

    def test_logging_out_with_cookie(self):
        "After logging out, a user can not use the cookie to authenticate anymore"
        self.test_login_without_header()
        with self.settings(DEBUG=True):
            engine = import_module(settings.SESSION_ENGINE)
            session = engine.SessionStore()

            session_id = self.client.session.session_key
            self.assertTrue(session.exists(session_id))

            response = self.delete('api-login')
            self.assertFalse(session.exists(session_id))

            response = self.get('api-login')

            self.assertEqual(response.status_code, 204)

    def test_can_not_start_authenticated_sessions_unauthenticated(self):
        "While anonymous session will just be started when not existing yet, authenticated ones can only be created by loggin in"
        with self.settings(DEBUG=True, SESSION_SAVE_EVERY_REQUEST=True):
            response = self.get('api-login', session_id='koe', authenticated=True)
            self.assertEqual(response.status_code, 401)


class SessionTest(APITest):
    def test_get_session(self):
        """
        Even when a session has expired, get_session should return a session
        with the requested id
        """
        with self.settings(SESSION_ENGINE='django.contrib.sessions.backends.db'):
            self._run_expired_session_test_for_engine()
        with self.settings(SESSION_ENGINE='django.contrib.sessions.backends.cache'):
            self._run_expired_session_test_for_engine()
        with self.settings(SESSION_ENGINE='django.contrib.sessions.backends.file'):
            self._run_expired_session_test_for_engine()
        with self.settings(SESSION_ENGINE='django.contrib.sessions.backends.cached_db'):
            self._run_expired_session_test_for_engine()

    def _run_expired_session_test_for_engine(self):
        # establish that get_session will return the same session
        # when that session key has not yet expired.
        session = get_session('session1')
        self.assertEqual(session.session_key, 'session1')
        session['touched'] = 'writesomething'
        session.save()
        session = get_session('session1')
        self.assertEqual(session.session_key, 'session1')
        session['touched'] = 'writesomethingelse'
        session.save()
        self.assertEqual(session.session_key, 'session1')

        # when the session expires immediately, the same session should
        # still be returned.
        # create a new session
        session = get_session('session2')
        self.assertEqual(session.session_key, 'session2')
        session['touched'] = 'writesomething'
        session.save()
        self.assertEqual(session.session_key, 'session2')

        # expire the session manually
        session.set_expiry(-100000)

        # get a session with the same id, even when it has expired.
        session = get_session('session2')
        self.assertEqual(session.session_key, 'session2')
        session['touched'] = 'writesomethingelse'
        session.save()
        self.assertEqual(session.session_key, 'session2')



from rest_framework import serializers
from rest_framework.fields import get_attribute


class TaxIncludedDecimalField(serializers.DecimalField):
    def __init__(self, excl_tax_field=None, excl_tax_value=None,
                 *args, **kwargs):
        self.excl_tax_field = excl_tax_field
        self.excl_tax_value = excl_tax_value
        super(TaxIncludedDecimalField, self).__init__(*args, **kwargs)

    def get_attribute(self, instance):
        if instance.is_tax_known:
            return super(TaxIncludedDecimalField, self).get_attribute(instance)
        if self.excl_tax_field:
            return get_attribute(instance, (self.excl_tax_field, ))
        return self.excl_tax_value



from django.utils import timezone

from oscar.core.loading import get_model

from oscarapi.tests.utils import APITest


Basket = get_model('basket', 'Basket')
ConditionalOffer = get_model('offer', 'ConditionalOffer')
Voucher = get_model('voucher', 'Voucher')


class VoucherTest(APITest):
    fixtures = [
        'product', 'productcategory', 'productattribute', 'productclass',
        'productattributevalue', 'category', 'attributeoptiongroup',
        'attributeoption', 'stockrecord', 'partner', 'voucher'
    ]

    def setUp(self):
        # Adjust offer dates so it's valid
        offer = ConditionalOffer.objects.get(
            name="Offer for voucher 'testvoucher'")
        offer.start_datetime = timezone.now()
        offer.end_datetime = timezone.now() + timezone.timedelta(days=1)
        offer.save()

        # adjust voucher dates for testing the view
        voucher = Voucher.objects.get(name="testvoucher")
        voucher.start_datetime = timezone.now()
        voucher.end_datetime = timezone.now() + timezone.timedelta(days=1)
        voucher.save()

        super(VoucherTest, self).setUp()

    def test_basket_add_voucher(self):
        """Check if we can add a voucher with the add-voucher api call"""
        # first add two products to our basket
        self.response = self.post(
            'api-basket-add-product',
            url="http://testserver/api/products/1/",
            quantity=2)
        self.response.assertStatusEqual(200)

        # total should be 20
        self.response = self.get('api-basket')
        self.response.assertValueEqual('total_incl_tax', '20.00')

        # add a voucher and see if the voucher is added correctly
        self.response = self.post(
            'api-basket-add-voucher',
            vouchercode='TESTVOUCHER')
        self.response.assertStatusEqual(200)

        # see if the discount of 5.00 from the voucher was applied
        self.response = self.get('api-basket')
        self.response.assertValueEqual('total_incl_tax', '15.00')

    def test_lowercase_voucher(self):
        """Lowercase vouchers should be working as well"""
        # first add two products to our basket
        self.response = self.post(
            'api-basket-add-product',
            url="http://testserver/api/products/1/",
            quantity=2)
        self.response.assertStatusEqual(200)

        self.response = self.post(
            'api-basket-add-voucher',
            vouchercode='testvoucher')
        self.response.assertStatusEqual(200)



import warnings

from django.db import IntegrityError
from rest_framework.response import Response

from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils.translation import gettext as _
from oscar.core import prices
from oscar.core.loading import get_class, get_model
from rest_framework import exceptions, serializers

from oscarapi.basket.operations import (
    assign_basket_strategy,
)
from oscarapi.serializers import (
    VoucherSerializer,
    OfferDiscountSerializer
)
from oscarapi.utils import (
    OscarHyperlinkedModelSerializer,
    OscarModelSerializer,
    overridable
)
from oscarapi.serializers.fields import TaxIncludedDecimalField


OrderPlacementMixin = get_class('checkout.mixins', 'OrderPlacementMixin')
OrderTotalCalculator = get_class('checkout.calculators',
                                 'OrderTotalCalculator')
ShippingAddress = get_model('order', 'ShippingAddress')
BillingAddress = get_model('order', 'BillingAddress')
Order = get_model('order', 'Order')
OrderLine = get_model('order', 'Line')
OrderLineAttribute = get_model('order', 'LineAttribute')

Basket = get_model('basket', 'Basket')
Country = get_model('address', 'Country')
Repository = get_class('shipping.repository', 'Repository')

UserAddress = get_model('address', 'UserAddress')


class PriceSerializer(serializers.Serializer):
    currency = serializers.CharField(
        max_length=12, default=settings.OSCAR_DEFAULT_CURRENCY, required=False)
    excl_tax = serializers.DecimalField(
        decimal_places=2, max_digits=12, required=True)
    incl_tax = TaxIncludedDecimalField(
        excl_tax_field='excl_tax',
        decimal_places=2, max_digits=12, required=False)
    tax = TaxIncludedDecimalField(
        excl_tax_value='0.00',
        decimal_places=2, max_digits=12, required=False)


class CountrySerializer(OscarHyperlinkedModelSerializer):
    class Meta:
        model = Country
        fields = '__all__'


class ShippingAddressSerializer(OscarHyperlinkedModelSerializer):
    class Meta:
        model = ShippingAddress
        fields = '__all__'


class InlineShippingAddressSerializer(OscarModelSerializer):
    country = serializers.HyperlinkedRelatedField(
        view_name='country-detail', queryset=Country.objects)

    class Meta:
        model = ShippingAddress
        fields = '__all__'


class BillingAddressSerializer(OscarHyperlinkedModelSerializer):
    class Meta:
        model = BillingAddress
        fields = '__all__'


class InlineBillingAddressSerializer(OscarModelSerializer):
    country = serializers.HyperlinkedRelatedField(
        view_name='country-detail', queryset=Country.objects)

    class Meta:
        model = BillingAddress
        fields = '__all__'


class ShippingMethodSerializer(serializers.Serializer):
    code = serializers.CharField(max_length=128)
    name = serializers.CharField(max_length=128)
    price = serializers.SerializerMethodField('calculate_price')

    def calculate_price(self, obj):
        price = obj.calculate(self.context.get('basket'))
        return PriceSerializer(price).data


class OrderLineAttributeSerializer(OscarHyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='order-lineattributes-detail')

    class Meta:
        model = OrderLineAttribute
        fields = '__all__'


class OrderLineSerializer(OscarHyperlinkedModelSerializer):
    "This serializer renames some fields so they match up with the basket"

    url = serializers.HyperlinkedIdentityField(view_name='order-lines-detail')
    attributes = OrderLineAttributeSerializer(
        many=True, fields=('url', 'option', 'value'), required=False)
    price_currency = serializers.CharField(
        source='order.currency', max_length=12)
    price_excl_tax = serializers.DecimalField(
        decimal_places=2, max_digits=12, source='line_price_excl_tax')
    price_incl_tax = serializers.DecimalField(
        decimal_places=2, max_digits=12, source='line_price_incl_tax')
    price_incl_tax_excl_discounts = serializers.DecimalField(
        decimal_places=2, max_digits=12,
        source='line_price_before_discounts_incl_tax')
    price_excl_tax_excl_discounts = serializers.DecimalField(
        decimal_places=2, max_digits=12,
        source='line_price_before_discounts_excl_tax')

    class Meta:
        model = OrderLine
        fields = overridable('OSCAR_ORDERLINE_FIELD', default=[
            'attributes', 'url', 'product', 'stockrecord', 'quantity',
            'price_currency', 'price_excl_tax', 'price_incl_tax',
            'price_incl_tax_excl_discounts', 'price_excl_tax_excl_discounts',
            'order'])


class OrderOfferDiscountSerializer(OfferDiscountSerializer):
    name = serializers.CharField(source='offer_name')
    amount = serializers.DecimalField(decimal_places=2, max_digits=12)


class OrderVoucherOfferSerializer(OrderOfferDiscountSerializer):
    voucher = VoucherSerializer(required=False)


class OrderSerializer(OscarHyperlinkedModelSerializer):
    """
    The order serializer tries to have the same kind of structure as the
    basket. That way the same kind of logic can be used to display the order
    as the basket in the checkout process.
    """
    owner = serializers.HyperlinkedRelatedField(
        view_name='user-detail', read_only=True, source='user')
    lines = serializers.HyperlinkedIdentityField(
        view_name='order-lines-list')
    shipping_address = InlineShippingAddressSerializer(
        many=False, required=False)
    billing_address = InlineBillingAddressSerializer(
        many=False, required=False)

    payment_url = serializers.SerializerMethodField()
    offer_discounts = serializers.SerializerMethodField()
    voucher_discounts = serializers.SerializerMethodField()

    def get_offer_discounts(self, obj):
        qs = obj.basket_discounts.filter(offer_id__isnull=False)
        return OrderOfferDiscountSerializer(qs, many=True).data

    def get_voucher_discounts(self, obj):
        qs = obj.basket_discounts.filter(voucher_id__isnull=False)
        return OrderVoucherOfferSerializer(qs, many=True).data

    def get_payment_url(self, obj):
        try:
            return reverse('api-payment', args=(obj.pk,))
        except NoReverseMatch:
            msg = "You need to implement a view named 'api-payment' " \
                "which redirects to the payment provider and sets up the " \
                "callbacks."
            warnings.warn(msg)
            return msg

    class Meta:
        model = Order
        fields = overridable('OSCARAPI_ORDER_FIELD', default=(
            'number', 'basket', 'url', 'lines',
            'owner', 'billing_address', 'currency', 'total_incl_tax',
            'total_excl_tax', 'shipping_incl_tax', 'shipping_excl_tax',
            'shipping_address', 'shipping_method', 'shipping_code', 'status',
            'guest_email', 'date_placed', 'payment_url', 'offer_discounts',
            'voucher_discounts')
        )


class CheckoutSerializer(serializers.Serializer, OrderPlacementMixin):
    basket = serializers.HyperlinkedRelatedField(
        view_name='basket-detail', queryset=Basket.objects)
    guest_email = serializers.EmailField(allow_blank=True, required=False)
    total = serializers.DecimalField(
        decimal_places=2, max_digits=12, required=False)
    shipping_method_code = serializers.CharField(
        max_length=128, required=False)
    shipping_charge = PriceSerializer(many=False, required=False)
    shipping_address = ShippingAddressSerializer(many=False, required=False)
    billing_address = BillingAddressSerializer(many=False, required=False)

    def get_initial_order_status(self, basket):
        return overridable('OSCARAPI_INITIAL_ORDER_STATUS', default='new')

    def validate(self, attrs):
        request = self.context['request']

        if request.user.is_anonymous():
            if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
                message = _('Anonymous checkout forbidden')
                raise serializers.ValidationError(message)

            if not attrs.get('guest_email'):
                # Always require the guest email field if the user is anonymous
                message = _('Guest email is required for anonymous checkouts')
                raise serializers.ValidationError(message)
        else:
            if 'guest_email' in attrs:
                # Don't store guest_email field if the user is authenticated
                del attrs['guest_email']

        basket = attrs.get('basket')
        basket = assign_basket_strategy(basket, request)
        if basket.num_items <= 0:
            message = _('Cannot checkout with empty basket')
            raise serializers.ValidationError(message)

        shipping_method = self._shipping_method(
            request, basket,
            attrs.get('shipping_method_code'),
            attrs.get('shipping_address')
        )
        shipping_charge = shipping_method.calculate(basket)
        posted_shipping_charge = attrs.get('shipping_charge')

        if posted_shipping_charge is not None:
            posted_shipping_charge = prices.Price(**posted_shipping_charge)
            # test submitted data.
            if not posted_shipping_charge == shipping_charge:
                message = _('Shipping price incorrect %s != %s' % (
                    posted_shipping_charge, shipping_charge
                ))
                raise serializers.ValidationError(message)

        posted_total = attrs.get('total')
        total = OrderTotalCalculator().calculate(basket, shipping_charge)
        if posted_total is not None:
            if posted_total != total.incl_tax:
                message = _('Total incorrect %s != %s' % (
                    posted_total,
                    total.incl_tax
                ))
                raise serializers.ValidationError(message)

        # update attrs with validated data.
        attrs['total'] = total
        attrs['shipping_method'] = shipping_method
        attrs['shipping_charge'] = shipping_charge
        attrs['basket'] = basket
        return attrs

    def create(self, validated_data):
        try:
            basket = validated_data.get('basket')
            order_number = self.generate_order_number(basket)
            request = self.context['request']

            shipping_address = ShippingAddress(
                **validated_data['shipping_address'])

            if 'billing_address' in validated_data:
                billing_address = BillingAddress(
                    **validated_data['billing_address'])
            else:
                billing_address = None

            return self.place_order(
                order_number=order_number,
                user=request.user,
                basket=basket,
                shipping_address=shipping_address,
                shipping_method=validated_data.get('shipping_method'),
                shipping_charge=validated_data.get('shipping_charge'),
                billing_address=billing_address,
                order_total=validated_data.get('total'),
                guest_email=validated_data.get('guest_email') or ''
            )
        except ValueError as e:
            raise exceptions.NotAcceptable(str(e))

    def _shipping_method(self, request, basket,
                         shipping_method_code, shipping_address):
        repo = Repository()

        default = repo.get_default_shipping_method(
            basket=basket,
            user=request.user,
            request=request,
            shipping_addr=shipping_address
        )

        if shipping_method_code is not None:
            methods = repo.get_shipping_methods(
                basket=basket,
                user=request.user,
                request=request,
                shipping_addr=shipping_address
            )

            find_method = (
                s for s in methods if s.code == shipping_method_code)
            shipping_method = next(find_method, default)
            return shipping_method

        return default


class UserAddressSerializer(OscarModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='useraddress-detail')
    country = serializers.HyperlinkedRelatedField(
        view_name='country-detail', queryset=Country.objects)

    def create(self, validated_data):
        request = self.context['request']
        validated_data['user'] = request.user
        try:
            return super(UserAddressSerializer, self).create(validated_data)
        except IntegrityError as e:
            raise exceptions.NotAcceptable(str(e))

    def update(self, instance, validated_data):
        # to be sure that we cannot change the owner of an address. If you
        # want this, please override the serializer
        request = self.context['request']
        validated_data['user'] = request.user
        try:
            return super(
                UserAddressSerializer, self).update(instance, validated_data)
        except IntegrityError as e:
            raise exceptions.NotAcceptable(str(e))

    class Meta:
        model = UserAddress
        fields = overridable('OSCARAPI_USERADDRESS_FIELDS', (
            'id', 'title', 'first_name', 'last_name', 'line1', 'line2',
            'line3', 'line4', 'state', 'postcode', 'search_text',
            'phone_number', 'notes', 'is_default_for_shipping',
            'is_default_for_billing', 'country', 'url'))



from django.conf.urls import include, url
from django.contrib import admin
from oscar.app import application
from oscarapi.app import application as api

urlpatterns = [
    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^api/', include(api.urls)),
    url(r'', include(application.urls)),
]



from django.core.exceptions import ValidationError

from oscar.core.loading import get_model

from oscarapi import permissions

from rest_framework import exceptions, generics
from rest_framework.relations import HyperlinkedRelatedField

__all__ = ('BasketPermissionMixin',)

Basket = get_model('basket', 'Basket')


def parse_basket_from_hyperlink(DATA, format):  # noqa
    "Parse basket from relation hyperlink"
    basket_parser = HyperlinkedRelatedField(
        view_name='basket-detail',
        queryset=Basket.objects,
        format=format
    )
    try:
        basket_uri = DATA.get('basket')
        data_basket = basket_parser.to_internal_value(basket_uri)
    except ValidationError as e:
        raise exceptions.NotAcceptable(e.messages)
    else:
        return data_basket


class BasketPermissionMixin(object):
    """
    This mixins adds some methods that can be used to check permissions
    on a basket instance.
    """
    # The permission class is mainly used to check Basket permission!
    permission_classes = (permissions.IsAdminUserOrRequestContainsBasket,)

    def get_data_basket(self, DATA, format):  # noqa
        return parse_basket_from_hyperlink(DATA, format)

    def check_basket_permission(self, request, basket_pk=None, basket=None):
        "Check if the user may access this basket"
        if basket is None:
            basket = generics.get_object_or_404(Basket.objects, pk=basket_pk)
        self.check_object_permissions(request, basket)
        return basket



from oscar.core.application import Application
from oscar.views.decorators import permissions_required

from oscarapi.urls import urlpatterns


class RESTApiApplication(Application):
    def get_url_decorator(self, pattern):
        "Fixes bug in Application.get_url_decorator wich would decorate None values."
        permissions = self.get_permissions(pattern.name)
        if permissions is not None:
            return permissions_required(permissions)

        return None

    def get_urls(self):
        return self.post_process_urls(urlpatterns)


application = RESTApiApplication()



from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

from oscar.apps.basket import signals
from oscar.core.loading import get_model, get_class

from rest_framework import status, generics, exceptions
from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.views import APIView

from oscarapi import serializers, permissions
from oscarapi.basket import operations
from oscarapi.views.mixin import PutIsPatchMixin
from oscarapi.views.utils import BasketPermissionMixin

__all__ = ('BasketView', 'LineList', 'LineDetail', 'AddProductView',
           'BasketLineDetail', 'AddVoucherView', 'shipping_methods')

Basket = get_model('basket', 'Basket')
Line = get_model('basket', 'Line')
Repository = get_class('shipping.repository', 'Repository')


class BasketView(APIView):
    """
    Api for retrieving a user's basket.

    GET:
    Retrieve your basket.
    """
    serializer_class = serializers.BasketSerializer

    def get(self, request, format=None):
        basket = operations.get_basket(request)
        ser = self.serializer_class(basket, context={'request': request})
        return Response(ser.data)


class AddProductView(APIView):
    """
    Add a certain quantity of a product to the basket.

    POST(url, quantity)
    {
        "url": "http://testserver.org/oscarapi/products/209/",
        "quantity": 6
    }

    If you've got some options to configure for the product to add to the
    basket, you should pass the optional ``options`` property:
    {
        "url": "http://testserver.org/oscarapi/products/209/",
        "quantity": 6,
        "options": [{
            "option": "http://testserver.org/oscarapi/options/1/",
            "value": "some value"
        }]
    }
    """
    add_product_serializer_class = serializers.AddProductSerializer
    serializer_class = serializers.BasketSerializer

    def validate(self, basket, product, quantity, options):
        availability = basket.strategy.fetch_for_product(
            product).availability

        # check if product is available at all
        if not availability.is_available_to_buy:
            return False, availability.message

        # check if we can buy this quantity
        allowed, message = availability.is_purchase_permitted(quantity)
        if not allowed:
            return False, message

        # check if there is a limit on amount
        allowed, message = basket.is_quantity_allowed(quantity)
        if not allowed:
            return False, message
        return True, None

    def post(self, request, format=None):
        p_ser = self.add_product_serializer_class(
            data=request.data, context={'request': request})
        if p_ser.is_valid():
            basket = operations.get_basket(request)
            product = p_ser.validated_data['url']
            quantity = p_ser.validated_data['quantity']
            options = p_ser.validated_data.get('options', [])

            basket_valid, message = self.validate(
                basket, product, quantity, options)
            if not basket_valid:
                return Response(
                    {'reason': message},
                    status=status.HTTP_406_NOT_ACCEPTABLE)

            basket.add_product(product, quantity=quantity, options=options)
            operations.apply_offers(request, basket)
            ser = self.serializer_class(
                basket, context={'request': request})
            return Response(ser.data)

        return Response(
            {'reason': p_ser.errors}, status=status.HTTP_406_NOT_ACCEPTABLE)


class AddVoucherView(APIView):
    """
    Add a voucher to the basket.

    POST(vouchercode)
    {
        "vouchercode": "kjadjhgadjgh7667"
    }

    Will return 200 and the voucher as json if succesful.
    If unsuccessful, will return 406 with the error.
    """
    add_voucher_serializer_class = serializers.VoucherAddSerializer
    serializer_class = serializers.VoucherSerializer

    def post(self, request, format=None):
        v_ser = self.add_voucher_serializer_class(
            data=request.data, context={'request': request})
        if v_ser.is_valid():
            basket = operations.get_basket(request)

            voucher = v_ser.instance
            basket.vouchers.add(voucher)

            signals.voucher_addition.send(
                sender=None, basket=basket, voucher=voucher)

            # Recalculate discounts to see if the voucher gives any
            operations.apply_offers(request, basket)
            discounts_after = basket.offer_applications

            # Look for discounts from this new voucher
            for discount in discounts_after:
                if discount['voucher'] and discount['voucher'] == voucher:
                    break
            else:
                basket.vouchers.remove(voucher)
                return Response(
                    {'reason': _(
                        "Your basket does not qualify for a voucher discount")},  # noqa
                    status=status.HTTP_406_NOT_ACCEPTABLE)

            ser = self.serializer_class(
                voucher, context={'request': request})
            return Response(ser.data)

        return Response(v_ser.errors, status=status.HTTP_406_NOT_ACCEPTABLE)


@api_view(('GET',))
def shipping_methods(request, format=None):
    """
    Get the available shipping methods and their cost for this order.

    GET:
    A list of shipping method details and the prices.
    """
    basket = operations.get_basket(request)
    shiping_methods = Repository().get_shipping_methods(
        basket=basket, user=request.user,
        request=request)
    ser = serializers.ShippingMethodSerializer(
        shiping_methods, many=True, context={'basket': basket})
    return Response(ser.data)


class LineList(BasketPermissionMixin, generics.ListCreateAPIView):
    """
    Api for adding lines to a basket.

    Permission will be checked,
    Regular users may only access their own basket,
    staff users may access any basket.

    GET:
    A list of basket lines.

    POST(basket, line_reference, product, stockrecord,
         quantity, price_currency, price_excl_tax, price_incl_tax):
    Add a line to the basket, example::

        {
            "basket": "http://127.0.0.1:8000/oscarapi/baskets/100/",
            "line_reference": "234_345",
            "product": "http://127.0.0.1:8000/oscarapi/products/209/",
            "stockrecord":
                "http://127.0.0.1:8000/oscarapi/stockrecords/100/",
            "quantity": 3,
            "price_currency": "EUR",
            "price_excl_tax": "100.0",
            "price_incl_tax": "121.0"
        }
    """
    serializer_class = serializers.LineSerializer
    queryset = Line.objects.all()

    def get_queryset(self):
        pk = self.kwargs.get('pk')
        if pk is not None:  # usually we need the lines of the basket
            basket = self.check_basket_permission(self.request, basket_pk=pk)
            prepped_basket = operations.assign_basket_strategy(
                basket, self.request)
            return prepped_basket.all_lines()
        elif self.request.user.is_staff:  # admin users can view a bit more
            return super(LineList, self).get_queryset()
        else:  # non admin users can view nothing at all here.
            return self.permission_denied(self.request)

    def get(self, request, pk=None, format=None):
        if pk is not None:
            basket = self.check_basket_permission(request, pk)
            prepped_basket = operations.assign_basket_strategy(basket, request)
            self.queryset = prepped_basket.all_lines()
            self.serializer_class = serializers.BasketLineSerializer

        return super(LineList, self).get(request, format)

    def post(self, request, pk=None, format=None):
        data_basket = self.get_data_basket(request.data, format)
        self.check_basket_permission(request, basket=data_basket)

        if pk is not None:
            url_basket = self.check_basket_permission(request, basket_pk=pk)
            if url_basket != data_basket:
                raise exceptions.NotAcceptable(
                    _('Target basket inconsistent %s != %s') % (
                        url_basket.pk, data_basket.pk
                    )
                )
        elif not request.user.is_staff:
            self.permission_denied(request)

        return super(LineList, self).post(request, format=format)


class LineDetail(PutIsPatchMixin, generics.RetrieveUpdateDestroyAPIView):
    queryset = Line.objects.all()
    serializer_class = serializers.LineSerializer
    permission_classes = (permissions.IsAdminUserOrRequestContainsLine,)

    def get(self, request, pk=None, format=None):
        line = self.get_object()
        basket = operations.get_basket(request)

        # if the line is from the current basket, use the serializer that
        # computes the prices by using the strategy.
        if line.basket == basket:
            operations.assign_basket_strategy(line.basket, request)
            ser = serializers.BasketLineSerializer(
                instance=line, context={'request': request})
            return Response(ser.data)

        return super(LineDetail, self).get(request, pk, format)


class BasketLineDetail(PutIsPatchMixin, generics.RetrieveUpdateDestroyAPIView):
    queryset = Line.objects.all()
    serializer_class = serializers.BasketLineSerializer
    permission_classes = (permissions.IsAdminUserOrRequestContainsLine,)

    def get_queryset(self):
        basket_pk = self.kwargs.get('basket_pk')
        basket = get_object_or_404(operations.editable_baskets(), pk=basket_pk)
        prepped_basket = operations.prepare_basket(basket, self.request)
        if operations.request_contains_basket(self.request, prepped_basket):
            return prepped_basket.all_lines()
        else:
            return self.queryset.none()



from django.conf import settings
from rest_framework import status
from rest_framework.exceptions import MethodNotAllowed
from rest_framework.response import Response
from rest_framework.views import APIView

from oscarapi import serializers
from oscarapi.utils import login_and_upgrade_session
from oscarapi.basket import operations
from oscar.core.loading import get_model


__all__ = ('LoginView',)

Basket = get_model('basket', 'Basket')


class LoginView(APIView):
    """
    Api for logging in users.

    DELETE:
    Log the user out by destroying the session.
    Anonymous users will have their cart destroyed as well, because there is
    no way to reach it anymoore

    POST(username, password):
    1. The user will be authenticated. The next steps will only be
       performed is login is succesful. Logging in logged in users results in
       405.
    2. The anonymous cart will be merged with the private cart associated with
       that authenticated user.
    3. A new session will be started, this session identifies the authenticated
       user for the duration of the session, without further need for
       authentication.
    4. The new, merged cart will be associated with this session.
    5. The anonymous session will be terminated.
    6. A response will be issued containing the new session id as a header
       (only when the request contained the session header as well).

    GET (enabled in DEBUG mode only):
    Get the details of the logged in user.
    If more details are needed, use the ``OSCARAPI_USER_FIELDS`` setting to change
    the fields the ``UserSerializer`` will render.
    """
    serializer_class = serializers.LoginSerializer

    def get(self, request, format=None):
        if settings.DEBUG:
            if request.user.is_authenticated():
                ser = serializers.UserSerializer(request.user, many=False)
                return Response(ser.data)
            return Response(status=status.HTTP_204_NO_CONTENT)

        raise MethodNotAllowed('GET')

    def merge_baskets(self, anonymous_basket, basket):
        "Hook to enforce rules when merging baskets."
        basket.merge(anonymous_basket)

    def post(self, request, format=None):
        ser = self.serializer_class(data=request.data)
        if ser.is_valid():

            anonymous_basket = operations.get_anonymous_basket(request)

            user = ser.instance

            # refuse to login logged in users, to avoid attaching sessions to
            # multiple users at the same time.
            if request.user.is_authenticated():
                return Response(
                    {'detail': 'Session is in use, log out first'},
                    status=status.HTTP_405_METHOD_NOT_ALLOWED)

            request.user = user

            login_and_upgrade_session(request._request, user)

            # merge anonymous basket with authenticated basket.
            basket = operations.get_user_basket(user)
            if anonymous_basket is not None:
                self.merge_baskets(anonymous_basket, basket)

            operations.store_basket_in_session(basket, request.session)

            return Response("")

        return Response(ser.errors, status=status.HTTP_401_UNAUTHORIZED)

    def delete(self, request, format=None):
        """
        Destroy the session.

        for anonymous users that means having their basket destroyed as well,
        because there is no way to reach it otherwise.
        """
        request = request._request
        if request.user.is_anonymous():
            basket = operations.get_anonymous_basket(request)
            if basket:
                operations.flush_and_delete_basket(basket)

        request.session.clear()
        request.session.delete()
        request.session = None

        return Response("")



from oscarapi.tests.utils import APITest
from django.core.urlresolvers import reverse


class ProductTest(APITest):
    fixtures = [
        'product', 'productcategory', 'productattribute', 'productclass',
        'productattributevalue', 'category', 'attributeoptiongroup',
        'attributeoption', 'stockrecord', 'partner'
    ]

    def test_product_list(self):
        "Check if we get a list of products with the default attributes"
        self.response = self.get('product-list')
        self.response.assertStatusEqual(200)
        # we should have two products
        self.assertEqual(len(self.response.body), 2)
        # default we have 3 fields
        product = self.response.body[0]
        default_fields = ['id', 'url']
        for field in default_fields:
            self.assertIn(field, product)

    def test_product_detail(self):
        "Check product details"
        self.response = self.get(reverse('product-detail', args=(1,)))
        self.response.assertStatusEqual(200)
        default_fields = ['stockrecords', 'description', 'title', 'url',
                          'date_updated', 'recommended_products', 'attributes',
                          'date_created', 'id', 'price', 'availability']
        for field in default_fields:
            self.assertIn(field, self.response.body)
        self.response.assertValueEqual('title', "Oscar T-shirt")

    def test_product_price(self):
        "See if we get the price information"
        self.response = self.get(reverse('product-price', args=(1,)))
        self.response.assertStatusEqual(200)
        self.assertIn('excl_tax', self.response.body)

        self.response = self.get(reverse('product-price', args=(999999,)))
        self.response.assertStatusEqual(404)

    def test_product_availability(self):
        "See if we get the availability information"
        self.response = self.get(reverse('product-availability', args=(1,)))
        self.response.assertStatusEqual(200)
        self.assertIn('num_available', self.response.body)

        self.response = self.get(reverse('product-availability', args=(999999,)))
        self.response.assertStatusEqual(404)



from oscar.core.loading import get_model
from oscarapi.basket.operations import request_contains_basket
from oscarapi.permissions import IsOwner
from oscarapi.serializers import (
    CheckoutSerializer, OrderLineAttributeSerializer, OrderLineSerializer,
    OrderSerializer, UserAddressSerializer
)
from oscarapi.signals import oscarapi_post_checkout
from oscarapi.views.utils import parse_basket_from_hyperlink
from rest_framework import generics, response, status, views

Order = get_model('order', 'Order')
OrderLine = get_model('order', 'Line')
OrderLineAttribute = get_model('order', 'LineAttribute')

UserAddress = get_model('address', 'UserAddress')

__all__ = (
    'CheckoutView',
    'OrderList', 'OrderDetail',
    'OrderLineList', 'OrderLineDetail',
    'OrderLineAttributeDetail',
    'UserAddressList',
    'UserAddressDetail'
)


class OrderList(generics.ListAPIView):
    serializer_class = OrderSerializer
    permission_classes = (IsOwner,)

    def get_queryset(self):
        qs = Order.objects.all()
        return qs.filter(user=self.request.user)


class OrderDetail(generics.RetrieveAPIView):
    queryset = Order.objects.all()
    serializer_class = OrderSerializer
    permission_classes = (IsOwner,)


class OrderLineList(generics.ListAPIView):
    queryset = OrderLine.objects.all()
    serializer_class = OrderLineSerializer

    def get(self, request, pk=None, format=None):
        if pk is not None:
            self.queryset = self.queryset.filter(
                order__id=pk, order__user=request.user)
        elif not request.user.is_staff:
            self.permission_denied(request)

        return super(OrderLineList, self).get(request, format)


class OrderLineDetail(generics.RetrieveAPIView):
    queryset = OrderLine.objects.all()
    serializer_class = OrderLineSerializer

    def get(self, request, pk=None, format=None):
        if not request.user.is_staff:
            self.queryset = self.queryset.filter(
                order__id=pk, order__user=request.user)

        return super(OrderLineDetail, self).get(request, format)


class OrderLineAttributeDetail(generics.RetrieveAPIView):
    queryset = OrderLineAttribute.objects.all()
    serializer_class = OrderLineAttributeSerializer


class CheckoutView(views.APIView):
    """
    Prepare an order for checkout.

    POST(basket, shipping_address,
         [total, shipping_method_code, shipping_charge, billing_address]):
    {
        "basket": "http://testserver/oscarapi/baskets/1/",
        "guest_email": "foo@example.com",
        "total": "100.0",
        "shipping_charge": {
            "currency": "EUR",
            "excl_tax": "10.0",
            "tax": "0.6"
        },
        "shipping_method_code": "no-shipping-required",
        "shipping_address": {
            "country": "http://127.0.0.1:8000/oscarapi/countries/NL/",
            "first_name": "Henk",
            "last_name": "Van den Heuvel",
            "line1": "Roemerlaan 44",
            "line2": "",
            "line3": "",
            "line4": "Kroekingen",
            "notes": "Niet STUK MAKEN OK!!!!",
            "phone_number": "+31 26 370 4887",
            "postcode": "7777KK",
            "state": "Gerendrecht",
            "title": "Mr"
        }
    }
    returns the order object.
    """
    order_serializer_class = OrderSerializer
    serializer_class = CheckoutSerializer

    def post(self, request, format=None):
        # TODO: Make it possible to create orders with options.
        # at the moment, no options are passed to this method, which means they
        # are also not created.

        basket = parse_basket_from_hyperlink(request.data, format)

        if not request_contains_basket(request, basket):
            return response.Response(
                "Unauthorized", status=status.HTTP_401_UNAUTHORIZED)

        c_ser = self.serializer_class(
            data=request.data, context={'request': request})

        if c_ser.is_valid():
            order = c_ser.save()
            basket.freeze()
            o_ser = self.order_serializer_class(
                order, context={'request': request})
            oscarapi_post_checkout.send(
                sender=self, order=order, user=request.user,
                request=request, response=response)
            return response.Response(o_ser.data)

        return response.Response(c_ser.errors, status.HTTP_406_NOT_ACCEPTABLE)


class UserAddressList(generics.ListCreateAPIView):
    serializer_class = UserAddressSerializer
    permission_classes = (IsOwner,)

    def get_queryset(self):
        return UserAddress.objects.filter(user=self.request.user)


class UserAddressDetail(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = UserAddressSerializer
    permission_classes = (IsOwner,)

    def get_queryset(self):
        return UserAddress.objects.filter(user=self.request.user)



"This module contains operation on baskets and lines"
from django.conf import settings
from oscar.core.loading import get_class, get_model
from oscar.core.utils import get_default_currency

__all__ = (
    'apply_offers',
    'assign_basket_strategy',
    'prepare_basket',
    'get_basket',
    'get_basket_id_from_session',
    'get_anonymous_basket',
    'get_user_basket',
    'store_basket_in_session',
    'request_contains_basket',
    'flush_and_delete_basket',
    'request_contains_line',
    'save_line_with_default_currency',
)

Basket = get_model('basket', 'Basket')
Applicator = get_class('offer.applicator', 'Applicator')
Selector = None


def apply_offers(request, basket):
    "Apply offers and discounts to cart"
    basket.reset_offer_applications()
    if not basket.is_empty:
        Applicator().apply(basket, request.user, request)


def assign_basket_strategy(basket, request):
    # fixes too early import of Selector
    # TODO: check if this is still true, now the basket models nolonger
    #       require this module to be loaded.
    global Selector

    if hasattr(request, 'strategy'):
        basket.strategy = request.strategy
    else:  # in management commands, the request might not be available.
        if Selector is None:
            Selector = get_class('partner.strategy', 'Selector')
        basket.strategy = Selector().strategy(
            request=request, user=request.user)

    apply_offers(request, basket)

    return basket


def prepare_basket(basket, request):
    assign_basket_strategy(basket, request)
    store_basket_in_session(basket, request.session)
    return basket


def get_basket(request, prepare=True):
    "Get basket from the request."
    if request.user.is_authenticated():
        basket = get_user_basket(request.user)
    else:
        basket = get_anonymous_basket(request)
        if basket is None:
            basket = Basket.objects.create()
            basket.save()
    return prepare_basket(basket, request) if prepare else basket


def get_basket_id_from_session(request):
    return request.session.get(settings.OSCAR_BASKET_COOKIE_OPEN)


def editable_baskets():
    return Basket.objects.filter(status__in=["Open", "Saved"])


def get_anonymous_basket(request):
    "Get basket from session."

    basket_id = get_basket_id_from_session(request)
    try:
        basket = editable_baskets().get(pk=basket_id)
    except Basket.DoesNotExist:
        basket = None

    return basket


def get_user_basket(user):
    "get basket for a user."

    try:
        basket, __ = editable_baskets().get_or_create(owner=user)
    except Basket.MultipleObjectsReturned:
        # Not sure quite how we end up here with multiple baskets.
        # We merge them and create a fresh one
        old_baskets = list(editable_baskets().filter(owner=user))
        basket = old_baskets[0]
        for other_basket in old_baskets[1:]:
            basket.merge(other_basket, add_quantities=False)

    return basket


def store_basket_in_session(basket, session):
    session[settings.OSCAR_BASKET_COOKIE_OPEN] = basket.pk
    session.save()


def request_contains_basket(request, basket):
    if basket.can_be_edited:
        if request.user.is_authenticated():
            return request.user == basket.owner

        return get_basket_id_from_session(request) == basket.pk

    return False


def flush_and_delete_basket(basket, using=None):
    "Delete basket and all lines"
    basket.flush()
    basket.delete(using)


def request_contains_line(request, line):
    basket = get_basket(request, prepare=False)
    if basket and basket.pk == line.basket.pk:
        return request_contains_basket(request, basket)
    return False


def save_line_with_default_currency(line, *args, **kwargs):
    if not line.price_currency:
        line.price_currency = get_default_currency()
    return line.save(*args, **kwargs)



import json
import re

from django.conf import settings
from django.core.urlresolvers import reverse

from oscar.core.loading import get_model

from oscarapi.tests.utils import APITest


Basket = get_model('basket', 'Basket')


class BasketTest(APITest):
    fixtures = [
        'product', 'productcategory', 'productattribute', 'productclass',
        'productattributevalue', 'category', 'attributeoptiongroup', 'attributeoption',
        'stockrecord', 'partner'
    ]

    def setUp(self):
        # make sure we have this disabled for most of the tests
        settings.OSCAR_MAX_BASKET_QUANTITY_THRESHOLD = None
        super(BasketTest, self).setUp()

    def test_basket_api_create(self):
        "The basket api create command should work with regular cookie based login"
        url = reverse('basket-list')
        empty = Basket.objects.all()
        self.assertFalse(empty.exists(), "There should be no baskets yet.")

        # anonymous
        data = {}

        self.response = self.client.post(url, json.dumps(data), content_type='application/json')
        self.response.assertStatusEqual(403, "Anonymous users can not use the basket api to create baskets.")

        # authenticated
        self.login('nobody', 'nobody')
        data = {'owner': "http://testserver%s" % reverse('user-detail', args=[2])}

        self.response = self.client.post(url, json.dumps(data), content_type='application/json')
        self.response.assertStatusEqual(403, "Authenticated regular users can not use the basket api to create baskets.")

        # admin
        self.login('admin', 'admin')

        data = {'owner': "http://testserver%s" % reverse('user-detail', args=[1])}
        self.response = self.client.post(url, json.dumps(data), content_type='application/json')
        self.response.assertStatusEqual(201, "It should be possible for a basket to be created, for a specific user.")
        self.response.assertObjectIdEqual('owner', 1)

        # When we created a basket, it should be listed in the basket-list view
        self.response = self.client.get(url, content_type='application/json')
        self.assertEqual(len(self.response.data), 1)

        data = {}
        self.response = self.client.post(url, json.dumps(data), content_type='application/json')
        self.response.assertStatusEqual(201, "It should be possible for a basket to be created for an anonymous user.")
        self.assertEqual(Basket.objects.count(), 2, "2 baskets should after creating 2 baskets.")

    def test_basket_api_create_header(self):
        "The basket api create command should work with header based login."
        empty = Basket.objects.all()
        self.assertFalse(empty.exists(), "There should be no baskets yet.")

        if self.hlogin('nobody', 'nobody', session_id='nobody'):
            self.response = self.post('basket-list', session_id='nobody', authenticated=True,
                owner="http://testserver%s" % reverse('user-detail', args=[2])
            )
            self.response.assertStatusEqual(403, "Authenticated regular users can not use the basket api to create baskets.")

        if self.hlogin('admin', 'admin', session_id='admin'):
            self.response = self.post('basket-list', session_id='admin', authenticated=True,
                owner="http://testserver%s" % reverse('user-detail', args=[1])
            )
            self.response.assertStatusEqual(201, "It should be possible for a basket to be created, for a specific user.")
            self.response.assertObjectIdEqual('owner', 1)

        self.assertEqual(Basket.objects.count(), 3, "There should be 2 baskets from loging in and 1 is created with the api.")
            
    def test_retrieve_basket(self):
        "A user can fetch their own basket with the basket API and get's the same basket every time."
        # anonymous
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('owner', None)
        basket_id = self.response['id']

        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        
        # authenticated
        self.login('nobody', 'nobody')
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        self.response.assertObjectIdEqual('owner', 2)
        basket_id = self.response['id']

        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)

        # admin
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            self.login('admin', 'admin')
            self.response = self.get('api-basket')
            self.response.assertStatusEqual(200)
            self.response.assertObjectIdEqual('owner', 1)
            basket_id = self.response['id']

            self.response = self.get('api-basket')
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id)

        self.assertEqual(Basket.objects.count(), 3, "There should be 3 baskets open after 3 users accessed a basket.")

    def test_retrieve_basket_header(self):
        "Using header authentication the basket api should also work perfectly."
        # anonymous
        self.response = self.get('api-basket', session_id='anonymous')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('owner', None)
        basket_id = self.response['id']

        self.response = self.get('api-basket', session_id='anonymous')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        
        # authenticated
        self.hlogin('nobody', 'nobody', session_id='nobody')
        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        self.response.assertObjectIdEqual('owner', 2)
        basket_id = self.response['id']

        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)

        # admin
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            self.hlogin('admin', 'admin', session_id='admin')
            self.response = self.get('api-basket', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)
            self.response.assertObjectIdEqual('owner', 1)
            basket_id = self.response['id']

            self.response = self.get('api-basket', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id)

        self.assertEqual(Basket.objects.count(), 3, "There should be 3 baskets open after 3 users accessed a basket.")

    def test_basket_read_permissions(self):
        "A regular or anonymous user should not be able to fetch someone elses basket."
        # anonymous user can retrive a basket.
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)

        # try to access the urls in the response.
        basket_url = self.response['url']
        basket_lines = self.response['lines']

        self.response = self.client.get(basket_url)
        self.response.assertStatusEqual(200)

        self.response = self.client.get(basket_lines)
        self.response.assertStatusEqual(200)

        # create a basket for somebody else
        b = Basket.objects.create(owner_id=2)
        self.assertEqual(str(b.owner), 'nobody')
        self.assertEqual(b.pk, 2)

        # try to acces somebody else's basket (hihi).
        url = reverse('basket-detail', args=(2,))
        self.response = self.client.get(url)
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users carts.")
        url = reverse('basket-lines-list', args=(2,))
        self.response = self.client.get(url)
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users cart items.")

        # now try for authenticated user.
        self.login('nobody', 'nobody')
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)

        # try to access the urls in the response.
        basket_url = self.response['url']
        basket_lines = self.response['lines']

        self.response = self.client.get(basket_url)
        self.response.assertStatusEqual(200)

        self.response = self.client.get(basket_lines)
        self.response.assertStatusEqual(200)

        # try to acces somebody else's basket (hihi).
        url = reverse('basket-detail', args=(1,))
        self.response = self.client.get(url)
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users carts.")

        url = reverse('basket-lines-list', args=(1,))
        self.response = self.client.get(url)
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users cart items.")

        # now let's show the power of the admin!
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            self.login('admin', 'admin')
            self.response = self.get('api-basket')
            self.response.assertStatusEqual(200)

            # try to access the urls in the response.
            basket_url = self.response['url']
            basket_lines = self.response['lines']

            self.response = self.client.get(basket_url)
            self.response.assertStatusEqual(200)

            self.response = self.client.get(basket_lines)
            self.response.assertStatusEqual(200)

            # try to acces somebody else's basket (hihi).
            url = reverse('basket-detail', args=(1,))
            self.response = self.client.get(url)
            self.response.assertStatusEqual(200, "Staff users can access anything.")

            url = reverse('basket-lines-list', args=(1,))
            self.response = self.client.get(url)
            self.response.assertStatusEqual(200, "Staff users can access anything.")

        self.assertEqual(Basket.objects.count(), 3, "There should be 3 baskets open after 3 users accessed a basket.")

    def test_basket_read_permissions_header(self):
        "A regular or anonymous user should not be able to fetch someone elses basket, even when authenticating with a session header."
        # anonymous user can retrieve a basket.
        self.response = self.get('api-basket', session_id='anonymous')
        self.response.assertStatusEqual(200)

        # try to access the urls in the response.
        basket_url = self.response['url']
        basket_lines = self.response['lines']

        self.response = self.client.get(basket_url, HTTP_SESSION_ID='SID:ANON:testserver:anonymous')
        self.response.assertStatusEqual(200)

        self.response = self.client.get(basket_lines, HTTP_SESSION_ID='SID:ANON:testserver:anonymous')
        self.response.assertStatusEqual(200)

        # create a basket for somebody else
        b = Basket.objects.create(owner_id=2)
        self.assertEqual(str(b.owner), 'nobody')
        self.assertEqual(b.pk, 2)

        # try to acces somebody else's basket (hihi).
        url = reverse('basket-detail', args=(2,))
        self.response = self.client.get(url, HTTP_SESSION_ID='SID:ANON:testserver:anonymous')
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users carts.")

        url = reverse('basket-lines-list', args=(2,))
        self.response = self.client.get(url, HTTP_SESSION_ID='SID:ANON:testserver:anonymous')
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users cart items.")

        # now try for authenticated user.
        self.hlogin('nobody', 'nobody', session_id='nobody')
        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)

        # try to access the urls in the response.
        basket_url = self.response['url']
        basket_lines = self.response['lines']

        self.response = self.client.get(basket_url, HTTP_SESSION_ID='SID:AUTH:testserver:nobody')
        self.response.assertStatusEqual(200)

        self.response = self.client.get(basket_lines, HTTP_SESSION_ID='SID:AUTH:testserver:nobody')
        self.response.assertStatusEqual(200)

        # try to acces somebody else's basket (hihi).
        url = reverse('basket-detail', args=(1,))
        self.response = self.client.get(url, HTTP_SESSION_ID='SID:AUTH:testserver:nobody')
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users carts.")

        url = reverse('basket-lines-list', args=(1,))
        self.response = self.client.get(url, HTTP_SESSION_ID='SID:AUTH:testserver:nobody')
        self.response.assertStatusEqual(403, "Script kiddies should fail to collect other users cart items.")

        # now let's show the power of the admin!
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            self.hlogin('admin', 'admin', session_id='admin')
            self.response = self.get('api-basket', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)

            # try to access the urls in the response.
            basket_url = self.response['url']
            basket_lines = self.response['lines']

            self.response = self.client.get(basket_url, HTTP_SESSION_ID='SID:AUTH:testserver:admin')
            self.response.assertStatusEqual(200)

            self.response = self.client.get(basket_lines, HTTP_SESSION_ID='SID:AUTH:testserver:admin')
            self.response.assertStatusEqual(200)

            # try to acces somebody else's basket (hihi).
            url = reverse('basket-detail', args=(1,))
            self.response = self.client.get(url, HTTP_SESSION_ID='SID:AUTH:testserver:admin')
            self.response.assertStatusEqual(200, "Staff users can access anything.")

            url = reverse('basket-lines-list', args=(1,))
            self.response = self.client.get(url, HTTP_SESSION_ID='SID:AUTH:testserver:admin')
            self.response.assertStatusEqual(200, "Staff users can access anything.")

        self.assertEqual(Basket.objects.count(), 3, "There should be 3 baskets open after 3 users accessed a basket.")

    def test_basket_write_permissions_anonymous(self):
        "An anonymous user should not be able to change someone elses basket."

        # anonymous user
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('status', 'Open')

        # try to access the urls in the response.
        basket_id = self.response['id']
        basket_url = self.response['url']

        # change status to saved
        url = reverse('basket-detail', args=(basket_id,))
        self.response = self.put(url, status='Saved')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        self.response.assertValueEqual('status', 'Saved')

        # and back to open again
        self.response = self.put(url, status='Open')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        self.response.assertValueEqual('status', 'Open')

        # write a line to the basket
        line_data = {
            "basket": basket_url, 
            "line_reference": "234_345", 
            "product": "http://testserver/api/products/1/", 
            "stockrecord": "http://testserver/api/stockrecords/1/", 
            "quantity": 3, 
            "price_currency": "EUR", 
            "price_excl_tax": "100.0", 
            "price_incl_tax": "121.0",
        }
        line_url = reverse('basket-lines-list', args=(basket_id,))
        self.response = self.post(line_url, **line_data)
        self.response.assertStatusEqual(201)

        # throw the basket away
        self.response = self.delete(url)
        self.response.assertStatusEqual(204)

        # now lets start messing around
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        basket_id = self.response['id']

        # create a basket for another user.
        b = Basket.objects.create(owner_id=2)
        self.assertEqual(str(b.owner), 'nobody')
        self.assertEqual(Basket.objects.count(), 2)
        nobody_basket_id = b.pk
        
        # try to access the urls in the response.
        basket_id = self.response['id']
        basket_url = self.response['url']
        url = reverse('basket-detail', args=(basket_id,))

        self.response.assertValueEqual('status', 'Open')

        # try to write to someone else's basket by sending the primary key
        # along.
        self.response = self.put(url, status='Saved', id=nobody_basket_id)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id, 'Primary key value can not be changed.')
        self.response.assertValueEqual('status', 'Saved')

        # try to write to someone else's basket directly
        url = reverse('basket-detail', args=(nobody_basket_id,))
        self.response = self.put(url, status='Saved')
        self.response.assertStatusEqual(403)

        # try to delete someone else's basket
        self.response = self.delete(url)
        self.response.assertStatusEqual(403)
        
        # try adding lines to someone elses basket
        line_data = {
            "basket": "http://testserver/api/baskets/%s/" % nobody_basket_id, 
            "line_reference": "234_345", 
            "product": "http://testserver/api/products/1/", 
            "stockrecord": "http://testserver/api/stockrecords/1/", 
            "quantity": 3, 
            "price_currency": "EUR", 
            "price_excl_tax": "100.0", 
            "price_incl_tax": "121.0"
        }
        url = reverse('basket-lines-list', args=(basket_id,))
        self.response = self.post(url, **line_data)
        self.response.assertStatusEqual(403)

    def test_basket_write_permissions_authenticated(self):
        "An authenticated user should not be able to change someone elses basket"

        # now try for authenticated user.
        self.login('nobody', 'nobody')
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)

        # try to access the urls in the response.
        basket_id = self.response['id']
        basket_url = self.response['url']
        owner_url = self.response['owner']
        self.assertIn(reverse('user-detail', args=(2,)), owner_url)
        self.response.assertValueEqual('status', 'Open')

        # change status to saved
        url = reverse('basket-detail', args=(basket_id,))
        self.response = self.put(url, status='Saved')

        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        self.response.assertValueEqual('status', 'Saved')

        # and back to open again
        self.response = self.put(url, status='Open')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        self.response.assertValueEqual('status', 'Open')

        # write a line to the basket
        line_data = {
            "basket": basket_url, 
            "line_reference": "234_345", 
            "product": "http://testserver/api/products/1/", 
            "stockrecord": "http://testserver/api/stockrecords/1/", 
            "quantity": 3, 
            "price_currency": "EUR", 
            "price_excl_tax": "100.0", 
            "price_incl_tax": "121.0",
        }
        line_url = reverse('basket-lines-list', args=(basket_id,))
        self.response = self.post(line_url, **line_data)
        self.response.assertStatusEqual(201)

        # throw the basket away
        self.response = self.delete(url)
        self.response.assertStatusEqual(204)

        # now lets start messing around
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        basket_id = self.response['id']

        # create a basket for another user.
        b = Basket.objects.create(owner_id=3)
        self.assertEqual(str(b.owner), 'somebody')
        self.assertEqual(Basket.objects.count(), 2)
        somebody_basket_id = b.pk

        # try to access the urls in the response.
        basket_id = self.response['id']
        basket_url = self.response['url']
        url = reverse('basket-detail', args=(basket_id,))

        self.response.assertValueEqual('status', 'Open')

        # try to write to someone else's basket by sending the primary key
        # along.
        self.response = self.put(url, status='Saved', id=somebody_basket_id)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id, 'Primary key value can not be changed.')
        self.response.assertValueEqual('status', 'Saved')

        # try to write to someone else's basket directly
        url = reverse('basket-detail', args=(somebody_basket_id,))
        self.response = self.put(url, status='Saved')
        self.response.assertStatusEqual(403)

        # try to delete someone else's basket
        self.response = self.delete(url)
        self.response.assertStatusEqual(403)

        # try adding lines to someone elses basket
        line_data = {
            "basket": "http://testserver/api/baskets/%s/" % somebody_basket_id,
            "line_reference": "234_345",
            "product": "http://testserver/api/products/1/",
            "stockrecord": "http://testserver/api/stockrecords/1/",
            "quantity": 3,
            "price_currency": "EUR",
            "price_excl_tax": "100.0",
            "price_incl_tax": "121.0"
        }
        url = reverse('basket-lines-list', args=(basket_id,))
        self.response = self.post(url, **line_data)
        self.response.assertStatusEqual(403)

    def test_basket_write_permissions_header_authenticated(self):
        "An authenticated user should not be able to change someone elses basket, when authinticating with session header."

        # now try for authenticated user.
        self.hlogin('nobody', 'nobody', session_id='nobody')
        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)

        # try to access the urls in the response.
        basket_id = self.response['id']
        basket_url = self.response['url']
        owner_url = self.response['owner']
        self.assertIn(reverse('user-detail', args=(2,)), owner_url)
        self.response.assertValueEqual('status', 'Open')

        # change status to saved
        url = reverse('basket-detail', args=(basket_id,))
        self.response = self.put(url, status='Saved', session_id='nobody', authenticated=True)

        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        self.response.assertValueEqual('status', 'Saved')

        # and back to open again
        self.response = self.put(url, status='Open', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id)
        self.response.assertValueEqual('status', 'Open')

        # write a line to the basket
        line_data = {
            "basket": basket_url, 
            "line_reference": "234_345", 
            "product": "http://testserver/api/products/1/", 
            "stockrecord": "http://testserver/api/stockrecords/1/", 
            "quantity": 3, 
            "price_currency": "EUR", 
            "price_excl_tax": "100.0", 
            "price_incl_tax": "121.0",
        }
        line_url = reverse('basket-lines-list', args=(basket_id,))
        self.response = self.post(line_url, session_id='nobody', authenticated=True, **line_data)
        self.response.assertStatusEqual(201)

        # throw the basket away
        self.response = self.delete(url, session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(204)

        # now lets start messing around
        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        basket_id = self.response['id']

        # create a basket for another user.
        b = Basket.objects.create(owner_id=3)
        self.assertEqual(str(b.owner), 'somebody')
        self.assertEqual(Basket.objects.count(), 2)
        somebody_basket_id = b.pk

        # try to access the urls in the response.
        basket_id = self.response['id']
        basket_url = self.response['url']
        url = reverse('basket-detail', args=(basket_id,))

        self.response.assertValueEqual('status', 'Open')

        # try to write to someone else's basket by sending the primary key
        # along.
        self.response = self.put(url, status='Saved', session_id='nobody', authenticated=True, id=somebody_basket_id)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('id', basket_id, 'Primary key value can not be changed.')
        self.response.assertValueEqual('status', 'Saved')

        # try to write to someone else's basket directly
        url = reverse('basket-detail', args=(somebody_basket_id,))
        self.response = self.put(url, status='Saved', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(403)

        # try to delete someone else's basket
        self.response = self.delete(url, session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(403)

        # try adding lines to someone elses basket
        line_data = {
            "basket": "http://testserver/api/baskets/%s/" % somebody_basket_id,
            "line_reference": "234_345",
            "product": "http://testserver/api/products/1/",
            "stockrecord": "http://testserver/api/stockrecords/1/",
            "quantity": 3,
            "price_currency": "EUR",
            "price_excl_tax": "100.0",
            "price_incl_tax": "121.0"
        }
        url = reverse('basket-lines-list', args=(basket_id,))
        self.response = self.post(url, session_id='nobody', authenticated=True, **line_data)
        self.response.assertStatusEqual(403)

    def test_basket_write_permissions_admin(self):
        "An admin user can change someone elses basket."

        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            # now try for authenticated user.
            self.login('admin', 'admin')
            self.response = self.get('api-basket')
            self.response.assertStatusEqual(200)

            # try to access the urls in the response.
            basket_id = self.response['id']
            basket_url = self.response['url']
            owner_url = self.response['owner']
            self.assertIn(reverse('user-detail', args=(1,)), owner_url)
            self.response.assertValueEqual('status', 'Open')

            # change status to saved
            url = reverse('basket-detail', args=(basket_id,))
            self.response = self.put(url, status='Saved')

            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id)
            self.response.assertValueEqual('status', 'Saved')

            # and back to open again
            self.response = self.put(url, status='Open')
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id)
            self.response.assertValueEqual('status', 'Open')

            # write a line to the basket
            line_data = {
                "basket": basket_url,
                "line_reference": "234_345",
                "product": "http://testserver/api/products/1/",
                "stockrecord": "http://testserver/api/stockrecords/1/",
                "quantity": 3,
                "price_currency": "EUR",
                "price_excl_tax": "100.0",
                "price_incl_tax": "121.0",
            }
            line_url = reverse('basket-lines-list', args=(basket_id,))
            self.response = self.post(line_url, **line_data)
            self.response.assertStatusEqual(201)

            # throw the basket away
            self.response = self.delete(url)
            self.response.assertStatusEqual(204)

            # now lets start messing around
            self.response = self.get('api-basket')
            self.response.assertStatusEqual(200)
            basket_id = self.response['id']

            # create a basket for another user.
            b = Basket.objects.create(owner_id=3)
            self.assertEqual(str(b.owner), 'somebody')
            self.assertEqual(Basket.objects.count(), 2)
            somebody_basket_id = b.pk

            # try to access the urls in the response.
            basket_id = self.response['id']
            basket_url = self.response['url']
            url = reverse('basket-detail', args=(basket_id,))

            self.response.assertValueEqual('status', 'Open')

            # try to write to someone else's basket by sending the primary key
            # along.
            self.response = self.put(url, status='Saved', id=somebody_basket_id)
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id, 'Primary key value can not be changed.')
            self.response.assertValueEqual('status', 'Saved')

            # try to write to someone else's basket directly
            url = reverse('basket-detail', args=(somebody_basket_id,))
            self.response = self.put(url, status='Saved')
            self.response.assertStatusEqual(200)

            # try adding lines to someone elses basket
            line_data = {
                "basket": "http://testserver/api/baskets/%s/" % somebody_basket_id,
                "line_reference": "234_345",
                "product": "http://testserver/api/products/1/",
                "stockrecord": "http://testserver/api/stockrecords/1/",
                "quantity": 3,
                "price_currency": "EUR",
                "price_excl_tax": "100.0",
                "price_incl_tax": "121.0"
            }
            zurl = reverse('basket-lines-list', args=(basket_id,))
            self.response = self.post(zurl, **line_data)
            self.response.assertStatusEqual(406)

            # try to delete someone else's basket
            self.response = self.delete(url)
            self.response.assertStatusEqual(204)

    def test_basket_write_permissions_header_admin(self):
        "An admin user can change someone elses basket, when authinticating with session header."
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            # now try for authenticated user.
            self.hlogin('admin', 'admin', session_id='admin')
            self.response = self.get('api-basket', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)

            # try to access the urls in the response.
            basket_id = self.response['id']
            basket_url = self.response['url']
            owner_url = self.response['owner']
            self.assertIn(reverse('user-detail', args=(1,)), owner_url)
            self.response.assertValueEqual('status', 'Open')

            # change status to saved
            url = reverse('basket-detail', args=(basket_id,))
            self.response = self.put(url, status='Saved', session_id='admin', authenticated=True)

            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id)
            self.response.assertValueEqual('status', 'Saved')

            # and back to open again
            self.response = self.put(url, status='Open', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id)
            self.response.assertValueEqual('status', 'Open')

            # write a line to the basket
            line_data = {
                "basket": basket_url,
                "line_reference": "234_345",
                "product": "http://testserver/api/products/1/",
                "stockrecord": "http://testserver/api/stockrecords/1/",
                "quantity": 3,
                "price_currency": "EUR",
                "price_excl_tax": "100.0",
                "price_incl_tax": "121.0",
            }
            line_url = reverse('basket-lines-list', args=(basket_id,))
            self.response = self.post(line_url, session_id='admin', authenticated=True, **line_data)
            self.response.assertStatusEqual(201)

            # throw the basket away
            self.response = self.delete(url, session_id='admin', authenticated=True)
            self.response.assertStatusEqual(204)

            # now lets start messing around
            self.response = self.get('api-basket', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)
            basket_id = self.response['id']

            # create a basket for another user.
            b = Basket.objects.create(owner_id=3)
            self.assertEqual(str(b.owner), 'somebody')
            self.assertEqual(Basket.objects.count(), 2)
            somebody_basket_id = b.pk

            # try to access the urls in the response.
            basket_id = self.response['id']
            basket_url = self.response['url']
            url = reverse('basket-detail', args=(basket_id,))

            self.response.assertValueEqual('status', 'Open')

            # try to write to someone else's basket by sending the primary key
            # along.
            self.response = self.put(url, status='Saved', session_id='admin', authenticated=True, id=somebody_basket_id)
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('id', basket_id, 'Primary key value can not be changed.')
            self.response.assertValueEqual('status', 'Saved')

            # try to write to someone else's basket directly
            url = reverse('basket-detail', args=(somebody_basket_id,))
            self.response = self.put(url, status='Saved', session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)

            # try adding lines to someone elses basket
            line_data = {
                "basket": "http://testserver/api/baskets/%s/" % somebody_basket_id,
                "line_reference": "234_345",
                "product": "http://testserver/api/products/1/",
                "stockrecord": "http://testserver/api/stockrecords/1/",
                "quantity": 3,
                "price_currency": "EUR",
                "price_excl_tax": "100.0",
                "price_incl_tax": "121.0"
            }
            zurl = reverse('basket-lines-list', args=(basket_id,))
            self.response = self.post(zurl, session_id='admin', authenticated=True, **line_data)
            self.response.assertStatusEqual(406)

            # try to delete someone else's basket
            self.response = self.delete(url, session_id='admin', authenticated=True)
            self.response.assertStatusEqual(204)

    def test_add_product_anonymous(self):
        "Test if an anonymous user can add a product to his basket"
        self.response = self.post('api-basket-add-product', url="http://testserver/api/products/1/", quantity=5)
        self.response.assertStatusEqual(200)

        self.response = self.get(self.response['lines'])
        self.assertEqual(len(self.response.body), 1)
        line0 = self.response.body[0]
        self.assertEqual(line0['product'], "http://testserver/api/products/1/")
        self.assertEqual(line0['quantity'], 5)


    def test_add_product_authenticated(self):
        "Test if an authenticated user can add a product to his basket"
        self.login('nobody', 'nobody')
        self.response = self.post('api-basket-add-product', url="http://testserver/api/products/1/", quantity=5)
        self.response.assertStatusEqual(200)

        self.response = self.get(self.response['lines'])
        self.assertEqual(len(self.response.body), 1)
        line0 = self.response.body[0]
        self.assertEqual(line0['product'], "http://testserver/api/products/1/")
        self.assertEqual(line0['quantity'], 5)

    def test_basket_line_permissions(self):
        "A user's Basket lines can not be viewed by another user in any way (except admins)"
        self.login('nobody', 'nobody')
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        
        self.response = self.post('api-basket-add-product', url="http://testserver/api/products/1/", quantity=5)
        self.response = self.get(self.response['lines'])
        line0 = self.response.body[0]
        line0url = line0['url']

        self.response = self.get(line0url)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('product', "http://testserver/api/products/1/")
        self.response.assertValueEqual('quantity', 5)

        # now let's try to cheat
        self.login('somebody', 'somebody')
        self.response = self.get(line0url)
        self.response.assertStatusEqual(404)
        
        # admin can cheat, but he uses a different url
        line0id = re.search('(?P<id>\d+)/$', line0url).group('id')
        admin_line0url = reverse('line-detail', args=(line0id,))
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            self.login('admin', 'admin')
            self.response = self.get(admin_line0url)
            self.response.assertStatusEqual(200)

        # nobody can not cheat like admin
        self.login('somebody', 'somebody')
        self.response = self.get(admin_line0url)
        self.response.assertStatusEqual(403)

    def test_basket_line_permissions_header(self):
        "A user's Basket lines can not be viewed by another user in any way (except admins), even with header authetication"
        self.hlogin('nobody', 'nobody', session_id='nobody')
        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        
        self.response = self.post('api-basket-add-product', url="http://testserver/api/products/1/", quantity=5, session_id='nobody', authenticated=True)
        self.response = self.get(self.response['lines'], session_id='nobody', authenticated=True)
        line0 = self.response.body[0]
        line0url = line0['url']

        self.response = self.get(line0url, session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('product', "http://testserver/api/products/1/")
        self.response.assertValueEqual('quantity', 5)

        # now let's try to cheat
        self.hlogin('somebody', 'somebody', session_id='somebody')
        self.response = self.get(line0url, session_id='somebody', authenticated=True)
        self.response.assertStatusEqual(404)

        # admin can cheat, but he uses a different url
        line0id = re.search('(?P<id>\d+)/$', line0url).group('id')
        admin_line0url = reverse('line-detail', args=(line0id,))
        with self.settings(OSCARAPI_BLOCK_ADMIN_API_ACCESS=False):
            self.hlogin('admin', 'admin', session_id='admin')
            self.response = self.get(admin_line0url, session_id='admin', authenticated=True)
            self.response.assertStatusEqual(200)

        # nobody can not cheat like admin
        self.login('somebody', 'somebody')
        self.response = self.get(admin_line0url)
        self.response.assertStatusEqual(403)

    def test_frozen_basket_can_not_be_accessed(self):
        "Prove that frozen baskets can nolonger be accessed by the user."
        self.login('nobody', 'nobody')
        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('status', 'Open')

        # change status to saved
        url = reverse('basket-detail', args=(self.response['id'],))
        self.response = self.put(url, status='Frozen')
        self.response.assertValueEqual('status', 'Frozen')

        self.response = self.get(url)
        self.response.assertStatusEqual(403)

    def test_frozen_basket_can_not_be_accessed_header(self):
        "Prove that frozen baskets can nolonger be accessed by the user, even with header authentication"
        self.hlogin('nobody', 'nobody', session_id='nobody')
        self.response = self.get('api-basket', session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('status', 'Open')

        # change status to saved
        url = reverse('basket-detail', args=(self.response['id'],))
        self.response = self.put(url, status='Frozen', session_id='nobody', authenticated=True)
        self.response.assertValueEqual('status', 'Frozen')

        self.response = self.get(url, session_id='nobody', authenticated=True)
        self.response.assertStatusEqual(403)

    def test_header_login_does_not_cause_regular_login(self):
        "Prove that there is not a bug in the test client that logs a user in when doing hlogin."
        self.hlogin('nobody', 'nobody', session_id='nobody')
        with self.settings(DEBUG=True):
            self.response = self.get('api-login')
            self.response.assertStatusEqual(204)
            self.response = self.get('api-login', session_id='nobody', authenticated=True)
            self.response.assertStatusEqual(200)
            self.response.assertValueEqual('username', 'nobody')

    def test_add_product_limit_basket(self):
        """Test if an anonymous user cannot add more than two products to his
            basket when amount of baskets is limited
        """
        settings.OSCAR_MAX_BASKET_QUANTITY_THRESHOLD = 2
        self.response = self.post(
            'api-basket-add-product',
            url="http://testserver/api/products/1/",
            quantity=3)
        self.response.assertStatusEqual(406)

    def test_total_prices_anonymous(self):
        "Test if the prices calcualted by the basket are ok"
        self.response = self.post(
            'api-basket-add-product',
            url="http://testserver/api/products/1/",
            quantity=5)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('total_incl_tax', '50.00')

    def test_add_product_above_stock(self):
        """Test if an anonymous user cannot add more products to his
            basket when stock is not sufficient
        """
        self.response = self.post(
            'api-basket-add-product',
            url="http://testserver/api/products/1/",
            quantity=25)
        self.response.assertStatusEqual(406)

    def test_adjust_basket_line_quantity(self):
        """Test if we can update the quantity of a line"""
        self.response = self.post(
            'api-basket-add-product',
            url="http://testserver/api/products/1/",
            quantity=5)
        self.response.assertStatusEqual(200)

        self.response = self.get('api-basket')
        self.response.assertStatusEqual(200)

        # Get the basket lines, and update the quantity to 4
        self.response = self.get(self.response['lines'])
        basket_line_url = self.response.data[0]['url']
        self.response = self.put(basket_line_url, quantity=4)
        self.response.assertStatusEqual(200)

        # see if it's updated
        self.response = self.get(basket_line_url)
        self.response.assertStatusEqual(200)
        self.response.assertValueEqual('quantity', 4)



import collections

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.reverse import reverse


__all__ = ('api_root',)


def PUBLIC_APIS(r, f):
    return [
        ('login', reverse('api-login', request=r, format=f)),
        ('basket', reverse('api-basket', request=r, format=f)),
        ('basket-add-product', reverse('api-basket-add-product', request=r,
                                       format=f)),
        ('basket-add-voucher', reverse('api-basket-add-voucher', request=r,
                                       format=f)),
        ('basket-shipping-methods', reverse('api-basket-shipping-methods', request=r,
                                       format=f)),
        ('checkout', reverse('api-checkout', request=r, format=f)),
        ('orders', reverse('order-list', request=r, format=f)),
        ('products', reverse('product-list', request=r, format=f)),
        ('countries', reverse('country-list', request=r, format=f)),
    ]


def PROTECTED_APIS(r, f):
    return [
        ('baskets', reverse('basket-list', request=r, format=f)),
        ('lines', reverse('line-list', request=r, format=f)),
        ('lineattributes', reverse('lineattribute-list', request=r, format=f)),
        ('options', reverse('option-list', request=r, format=f)),
        ('stockrecords', reverse('stockrecord-list', request=r, format=f)),
        ('users', reverse('user-list', request=r, format=f)),
        ('partners', reverse('partner-list', request=r, format=f)),
        ('useraddresses', reverse('useraddress-list', request=r, format=f)),
    ]


@api_view(('GET',))
def api_root(request, format=None):
    """
    GET:
    Display all available urls.

    Since some urls have specific permissions, you might not be able to access
    them all.
    """
    apis = PUBLIC_APIS(request, format)
    if request.user.is_staff:
        apis += PROTECTED_APIS(request, format)

    return Response(collections.OrderedDict(apis))
